from twisted.internet.defer import inlineCallbacks, returnValue

from ensemble.control.utils import get_environment


def configure_subparser(subparsers):
    """Configure shutdown subcommand"""
    sub_parser = subparsers.add_parser("shutdown", help=command.__doc__)
    sub_parser.add_argument(
        "--environment", "-e",
        help="Ensemble environment to operate in.")
    return sub_parser


@inlineCallbacks
def command(options):
    """
    Shutdown machine providers in all configured environments.
    """
    environment = get_environment(options)
    provider = environment.get_machine_provider()

    value = raw_input(
        "Warning, this will destroy all machines and services in the\n"
        "environment. Continue [y/N]")

    if value.strip().lower() not in ("y", "yes"):
        options.log.info("Shutdown aborted")
        returnValue(None)
    options.log.info("Shutting down environment %r (type: %s)..." % (
        environment.name, environment.type))
    yield provider.shutdown()
