
from ensemble.errors import FormulaError


class FormulaIdError(FormulaError):
    """A formula id was invalid.
    """

    def __init__(self, formula_id, reason):
        self.formula_id = formula_id
        self.reason = reason

    def __str__(self):
        return "Invalid formula id %r %s" % (self.formula_id, self.reason)


class FormulaNotFound(FormulaError):
    """A formula was not found in the repository."""

    def __init__(self, repository_path, formula_name):
        self.repository_path = repository_path
        self.formula_name = formula_name

    def __str__(self):
        return "Formula '%s' not found in repository %s" % (
            self.formula_name, self.repository_path)


class InvalidFormulaHook(FormulaError):
    """A named hook was not found to be valid for the formula."""

    def __init__(self, formula_name, hook_name):
        self.formula_name = formula_name
        self.hook_name = hook_name

    def __str__(self):
        return "Formula %r does not contain hook %r" % (
            self.formula_name, self.hook_name)


class NewerFormulaNotFound(FormulaError):
    """A newer formula was not found."""

    def __init__(self, formula_id):
        self.formula_id = formula_id

    def __str__(self):
        return "Formula %r is the latest revision known" % self.formula_id


class ServiceConfigError(Exception):
    """Indicates an issue related to service options."""

