from twisted.internet.defer import fail

from ensemble.errors import ProviderInteractionError

from .iterate import EC2ProviderMachineFilter
from .machine import machine_from_instance


class EC2MachineAccessor(EC2ProviderMachineFilter):
    """Retrieve a specific machine by provider id."""

    def run(self, provider_id):
        d = self._provider.ec2.describe_instances(provider_id)
        d.addCallback(self._filter_provider_machines)
        d.addCallback(self._create_provider_machine, provider_id)
        return d

    def _create_provider_machine(self, instances, provider_id):
        if not instances:
            return fail(
                ProviderInteractionError(
                    "Machine (id: %r) was not found" % provider_id))

        instance = instances.pop()
        return machine_from_instance(instance)
