import datetime
from datetime import timedelta

from twisted.internet.defer import inlineCallbacks, returnValue

from ensemble.state.sshclient import SSHClient
from ensemble.errors import NoConnection

from .utils import log


class EC2Connect(object):

    def __init__(self, provider):
        self._provider = provider

    def run(self, share=False):
        client_deferred = self._provider.get_zookeeper_machines()
        client_deferred.addCallback(self._connect_to_machine, share)
        client_deferred.addCallback(self._wait_for_initialization)
        return client_deferred

    def _connect_to_machine(self, machines, share):
        # TODO Should we pick a random entry from the nodes list?
        log.info("Connecting to environment.")
        client = SSHClient()
        machine = machines[0]
        result = client.connect(machine.dns_name + ":2181",
                                timeout=30, share=share)
        result.addErrback(self._check_machine_age, machine)
        return result

    def _check_machine_age(self, failure, machine):
        failure.trap(NoConnection)
        five_minutes_ago = datetime.datetime.now() - timedelta(minutes=5)
        if machine.launch_time > five_minutes_ago:
            raise NoConnection("Can't yet connect to started machine: %s" %
                               str(failure.value))
        return failure

    @inlineCallbacks
    def _wait_for_initialization(self, client):
        exists_d, watch_d = client.exists_and_watch("/initialized")
        exists = yield exists_d
        if not exists:
            log.info("Environment still initializing.  Will wait.")
            yield watch_d
        else:
            log.debug("Environment already initialized.")
        returnValue(client)
