from twisted.internet.defer import succeed
from txaws.ec2.model import Instance, Reservation

from ensemble.errors import ProviderInteractionError
from ensemble.providers.ec2.machine import EC2ProviderMachine

from ensemble.lib.testing import TestCase
from .common import EC2TestMixin


class MachineAccessorTestCase(EC2TestMixin, TestCase):

    def test_machine_accessor(self):
        """
        Machines can be fetched by provider id.
        """
        group_name = "ensemble-%s" % self.env_name
        instances = [
            Instance(
                "i-foobar", "running", dns_name="x1.example.com",
                reservation=Reservation("x-1", "u-1", groups=[group_name]))]

        self.ec2.describe_instances("i-foobar")
        self.mocker.result(succeed(instances))
        self.mocker.replay()

        provider = self.get_provider()

        def validate_result(machine):
            self.assertTrue(machine)
            self.assertTrue(isinstance(machine, EC2ProviderMachine))
            self.assertEquals(machine.instance_id,
                              instances[0].instance_id)
            self.assertEquals(machine.dns_name,
                              instances[0].dns_name)

        d = provider.get_machine(instances[0].instance_id)
        d.addCallback(validate_result)
        return d

    def test_machine_not_found(self):

        instances = []

        self.ec2.describe_instances("i-foobar")
        self.mocker.result(succeed(instances))
        self.mocker.replay()

        provider = self.get_provider()

        d = self.assertFailure(
            provider.get_machine("i-foobar"),
            ProviderInteractionError)

        def validate_error(error):
            self.assertIn(
                "Machine (id: 'i-foobar') was not found",
                str(error))

        d.addCallback(validate_error)
        return d
