import os
import shutil
import tempfile
import subprocess
import re
import logging
import xmlrpclib
import base64, commands, sys, gzip, StringIO

from yaml import safe_dump, load

log = logging.getLogger("ensemble.provider.orchestra")

from ensemble.errors import (
    FileNotFound, ProviderError, ProviderInteractionError)

class CobblerControl(object):

    def __init__(self, config):
        self.config = config
        self._conn = xmlrpclib.Server('http://%(orchestra-server)s/cobbler_api'
                % self.config)
        self._token = self._conn.login(self.config['orchestra-user'],
                self.config['orchestra-pass'])


    def acquire_system(self, ks_meta={}):
        # list all systems
        for sys in self._conn.get_systems():
            if self.config['available-mgmt-class'] in sys['mgmt_classes']:
                if sys['netboot_enabled']:
                    handle = self._conn.get_system_handle(sys['name'], self._token)
                    if not self._conn.modify_system(handle, 
                            'mgmt_classes', 
                            [self.config['acquired-mgmt-class']],
                            self._token):
                        raise Exception("Could not modify mgmt_classes")
                    if not self._conn.modify_system(handle,
                            'ks_meta',ks_meta,
                            self._token):
                        raise Exception("Could not modify ks_meta")
                    if not self._conn.save_system(handle, self._token):
                        raise Exception("Could not save system")
                    return sys['name']
        raise Exception('Could not acquire any servers using mgmt_class=%s' 
                % self.config['available-mgmt-class'])


    def get_system_ks_meta(self, name):
        sys = self._conn.get_system(name)
        return sys['ks_meta']
        

    def set_system_ks_meta(self, name, ks_args):
        handle = self._conn.get_system_handle(name, self._token)
        if not self._conn.modify_system(handle, "ks_meta", ks_args, self._token):
            raise Exception("Could not modify ks_meta")
        if not self._conn.save_system(handle, self._token):
            raise Exception("Could not save system %s" % system['name'])


    def start_system(self, name):
        # set boot from network = True
        handle = self._conn.get_system_handle(name, self._token)
        if not self._conn.modify_system(handle, 'netboot_enabled', True, self._token):
            raise Exception("Could not modify netboot_enabled")
        if not self._conn.save_system(handle, self._token):
            raise Exception("Could not save system %s" % system['name'])
        # poweron system
        if not self._conn.background_power_system(
                {'power':'on','systems':[name]}, self._token):
            raise Exception("could not power on system")


    def release_system(self, name):
        handle = self._conn.get_system_handle(name, self._token)
        if not self._conn.background_power_system(
                {'power':'off','systems':[name]}, self._token):
            raise Exception("could not power off system")
        if not self._conn.modify_system(handle, 'mgmt_classes', 
                [self.config['available-mgmt-class']], self._token):
            raise Exception("Could not modify system %s" % name)
        if not self._conn.modify_system(handle, 'ks_meta', '', self._token):
            raise Exception("Could not modify system %s" % name)
        if not self._conn.save_system(handle, self._token):
            raise Exception("Could not save system %s" % name)


    def get_acquired_systems(self):
        # FIXME: This seems to be returning 3 machines
        for name in self._conn.find_system(
                {'mgmt_classes':self.config['acquired-mgmt-class']}):
            yield name

