from twisted.internet.defer import fail, inlineCallbacks, returnValue

from ensemble.providers.common.launch import LaunchMachine
from ensemble.providers.common.utils import format_cloud_ini

from .machine import OrchestraMachine

import base64, commands, sys, gzip, StringIO

import os

def get_launch_options(config, authorized_keys, packages=None,
                       repositories=None, scripts=None, **kw):
    user_data = format_cloud_ini(authorized_keys=authorized_keys,
                                 packages=packages,
                                 repositories=repositories,
                                 scripts=scripts,
                                 data=kw)
    return user_data

class OrchestraLaunchMachine(LaunchMachine):

    @inlineCallbacks
    def start_machine(self, variables):
        # 1. Acquire an available system.
        name = yield self._provider.control.acquire_system()

        # 2. Obtain the cloud-init config
        cloud_ini = get_launch_options(
            self._provider.config, **variables)

        # 3. Provide cloud-init-config file for base64 functions for late-command.
        late_command = self.get_cloud_init_late_command(name, cloud_ini)
        late_command_ksmeta = self.get_ks_meta_for_late_command(late_command)
        # NOTE: This adds the machine_id arg to ks_meta to not fail on obtaining machines
        late_command_ksmeta = late_command_ksmeta + " machine_id=%s " % variables.get('machine-id')

        # 5. Update the machine profile with late-command
        self._provider.control.set_system_ks_meta(name, late_command_ksmeta)

        # 6. start_machine
        self._provider.control.start_system(name)

        machine = OrchestraMachine(name, name)

        returnValue([machine])

    def get_ks_meta_for_late_command(self, contents, keyname="ENSEMBLE_LATE_COMMAND", compress=True):
        if compress:
            encIO = StringIO.StringIO()
            gzfp = gzip.GzipFile(filename=None, mode="wb", compresslevel=9, fileobj=encIO)
            gzfp.write(contents)
            gzfp.close()
            encoded = base64.b64encode(encIO.getvalue())
            encIO.close()
            pipe_gunzip = "| gunzip"
        else:
            encoded = base64.b64encode(contents)
            pipe_gunzip = ""
        command = "in-target sh -c 'f=$1; shift; echo $0 | base64 --decode %s > $f && chmod u+x $f && $f $*' %s /root/late-command" % (pipe_gunzip, encoded)
        #command = "in-target sh -c 'echo $0 | base64 --decode %s | sh -' %s" % (pipe_gunzip, encoded)
        return('%s="%s"' % (keyname, command))


    def get_cloud_init_late_command(self, name, cloud_ini):
        late_command = """
seed_d=/var/lib/cloud/seed/nocloud-net
mkdir -p "$seed_d"
cat > "$seed_d"/meta-data <<"EOF"
instance-id: %s
EOF

cat > "$seed_d"/user-data <<"EOF"
%s """ % (name, cloud_ini)
        return late_command
