import tempfile
from yaml import safe_dump, load
from twisted.internet.defer import succeed, fail

class OrchestraSaveState(object):

    def __init__(self, provider):
        self._provider = provider

    def run(self, state):
        storage = self._provider.get_file_storage()
        with tempfile.NamedTemporaryFile(delete=True) as statefile:
            statefile.write(safe_dump(state))
            statefile.flush()
            storage.put('__state__',statefile.name)
        return succeed(None)

class OrchestraLoadState(object):
    def __init__(self, provider):
        self._provider = provider

    def run(self):
        # load state
        def _on_not_found(failure):
            failure.trap(FileNotFound)
            return self.save_state({})
        storage = self._provider.get_file_storage()
        d = storage.get('__state__')
        d.addErrback(_on_not_found)
        d.addCallback(self._on_state_retrieved)
        return d

    def _on_state_retrieved(self, content):
        if not content:
            return False
        state = load(content)
        if not state:
            return False
        return state
